/**
 * A brief explanation for "project.json":
 * Here is the content of project.json file, this is the global configuration for your game, you can modify it to customize some behavior.
 * The detail of each field is under it.
 {
    "project_type": "javascript",
    // "project_type" indicate the program language of your project, you can ignore this field

    "debugMode"     : 1,
    // "debugMode" possible values :
    //      0 - No message will be printed.
    //      1 - cc.error, cc.assert, cc.warn, cc.log will print in console.
    //      2 - cc.error, cc.assert, cc.warn will print in console.
    //      3 - cc.error, cc.assert will print in console.
    //      4 - cc.error, cc.assert, cc.warn, cc.log will print on canvas, available only on web.
    //      5 - cc.error, cc.assert, cc.warn will print on canvas, available only on web.
    //      6 - cc.error, cc.assert will print on canvas, available only on web.

    "showFPS"       : true,
    // Left bottom corner fps information will show when "showFPS" equals true, otherwise it will be hide.

    "frameRate"     : 60,
    // "frameRate" set the wanted frame rate for your game, but the real fps depends on your game implementation and the running environment.

    "id"            : "gameCanvas",
    // "gameCanvas" sets the id of your canvas element on the web page, it's useful only on web.

    "renderMode"    : 0,
    // "renderMode" sets the renderer type, only useful on web :
    //      0 - Automatically chosen by engine
    //      1 - Forced to use canvas renderer
    //      2 - Forced to use WebGL renderer, but this will be ignored on mobile browsers

    "engineDir"     : "frameworks/cocos2d-html5/",
    // In debug mode, if you use the whole engine to develop your game, you should specify its relative path with "engineDir",
    // but if you are using a single engine file, you can ignore it.

    "modules"       : ["cocos2d"],
    // "modules" defines which modules you will need in your game, it's useful only on web,
    // using this can greatly reduce your game's resource size, and the cocos console tool can package your game with only the modules you set.
    // For details about modules definitions, you can refer to "../../frameworks/cocos2d-html5/modulesConfig.json".

    "jsList"        : [
    ]
    // "jsList" sets the list of js files in your game.
 }
 *
 */

cc.game.onStart = function(){
    if(!cc.sys.isNative && document.getElementById("cocosLoading")) //If referenced loading.js, please remove it
        document.body.removeChild(document.getElementById("cocosLoading"));

    // Pass true to enable retina display, disabled by default to improve performance
    cc.view.enableRetina(false);
    // Adjust viewport meta
    cc.view.adjustViewPort(true);
    // Setup the resolution policy and design resolution size
    //cc.view.setDesignResolutionSize(800, 450, cc.ResolutionPolicy.SHOW_ALL);
    // The game will be resized when browser size change
    cc.view.resizeWithBrowserSize(true);

    // auto resolution setting
    var designResSize = cc.view.getDesignResolutionSize();

    // Portrait
    //var targetWidth = 640;
    //var targetScreenRatio = 1.5;
    //var targetHeight = targetWidth * targetScreenRatio;
    //if( cc.sys.isNative ) {
    //    if( designResSize.height / designResSize.width >= targetScreenRatio ) {
    //        cc.view.setDesignResolutionSize(targetWidth
    //            , (designResSize.height / designResSize.width) * targetWidth
    //            , cc.ResolutionPolicy.SHOW_ALL);
    //    }
    //    else {
    //        cc.view.setDesignResolutionSize((designResSize.width / designResSize.height) * targetHeight
    //            , targetHeight
    //            , cc.ResolutionPolicy.SHOW_ALL);
    //    }
    //}
    //else {
    //    cc.view.setDesignResolutionSize(targetWidth
    //        , targetScreenRatio * targetWidth
    //        , cc.ResolutionPolicy.SHOW_ALL);
    //}

    // Landscape
    var targetHeight = 640;
    var targetScreenRatio = 1.5;
    var targetWidth = targetHeight * targetScreenRatio;
    if( cc.sys.isNative ) {
        if( designResSize.width / designResSize.height >= targetScreenRatio ) {
            cc.view.setDesignResolutionSize((designResSize.width / designResSize.height) * targetHeight
                , targetHeight
                , cc.ResolutionPolicy.SHOW_ALL);
        }
        else {
            cc.view.setDesignResolutionSize(targetWidth
                , (designResSize.height / designResSize.width) * targetWidth
                , cc.ResolutionPolicy.SHOW_ALL);
        }
    }
    else {
        cc.view.setDesignResolutionSize(targetScreenRatio * targetHeight
            , targetHeight
            , cc.ResolutionPolicy.SHOW_ALL);
    }

    //cc.log("designResSize.width: "+designResSize.width);
    //cc.log("designResSize.height: "+designResSize.height);
    //cc.log("ratio: "+(designResSize.width / designResSize.height));

    setupGlobalVariables();

    //load resources
    cc.LoaderScene.preload(g_resources, function () {
        cc.director.runScene(new PlayScene());
    }, this);

    g_systemManager.loadingToMainMenu();
    //g_playScene.delayedLoadingToMainMenu();
};
cc.game.run();

var winSize, winCenter;
var inputManager;
function setupGlobalVariables() {
    winSize = cc.director.getWinSize();
    winCenter = cc.p(winSize.width*0.5, winSize.height*0.5);
    inputManager = InputManager.getInstance();
}