#ifdef GL_ES
precision mediump float;
#endif

uniform sampler2D u_texture;
uniform vec3 u_targetColor;     // 효과를 줄 색상값.
uniform float u_percent;        // 효과의 정도 0~1
uniform float u_whiteMax;       // 이미지의 흰색에 적용할 가중치.

varying vec2 v_texCoord;
varying vec4 v_fragmentColor;

void main(void)
{
    vec4 color = texture2D(CC_Texture0, v_texCoord);
    if ( color.a > 0.0 ) {
        if (color.a != 1.0) {
            // 알파값이 1이 아닌경우, rgb 값에 알파값이 곱해진 색상이 들어오므로,
            // 원본 rgb 를 구하려면 알파값으로 나눠야 한다.
            // 그리고 적용할 색상을 구한후 다시 알파값을 곱해준다.
            color.rgb = color.rgb / color.a;

            // 명도값을 구하고, 그에 따른 가중치를 구하고,
            // 거기에 효과를 줄 정도값을 곱하여 최종 색상적용 percent 값을 구한다.
            float percent;
            float L = (max(max(color.r, color.g), color.b) + min(min(color.r, color.g), color.b)) * 0.5;
            if (L < 0.5) {
                percent = u_percent * (L * 2.0);
            }
            else {
                percent = u_percent * (2.0*(u_whiteMax-1.0)*L + (2.0 - u_whiteMax));
            }

            // 위에서 구한 값을 사용하여 최종 색상값을 구한다.
            color.rgb = vec3(color.r + (u_targetColor.r-color.r) * percent
                                , color.g + (u_targetColor.g-color.g) * percent
                                , color.b + (u_targetColor.b-color.b) * percent
                                ) * color.a;
            gl_FragColor = color * v_fragmentColor;
        }
        else {
            float percent;
            float L = (max(max(color.r, color.g), color.b) + min(min(color.r, color.g), color.b)) * 0.5;
            if (L < 0.5) {
                percent = u_percent * (L * 2.0);
            }
            else {
                percent = u_percent * (2.0*(u_whiteMax-1.0)*L + (2.0 - u_whiteMax));
            }
            
            gl_FragColor = vec4(color.r + (u_targetColor.r-color.r) * percent
                                , color.g + (u_targetColor.g-color.g) * percent
                                , color.b + (u_targetColor.b-color.b) * percent
                                , color.a) * v_fragmentColor;
        }
    }
    else {
        gl_FragColor = color;
    }
}
