/**
 * Created by kominpyo on 15. 7. 22..
 */

var g_ballManager;

var BallManager = cc.Node.extend({

    ctor: function() {
        this._super();
        g_ballManager = this;

        // 풀 셋업
        for( var i = 30; i--; ) {
            var o = new Ball();
            cc.pool.putInPool(o);
        }

        this.activeObjects = [];

        // 공 색상 순차랜덤.
        this.seqRandBallColor = new SequencedRandom(5);

        this.overGaugeHeightFromLand = winSize.height - LAND_POS_Y -25; // 25: 게이지 + TopSign 배경.
    },

    mainMenuToPlay: function() {
        // 게임 시작 초기화.
        this.resetGameStart();
    },

    playToPlayEnd: function() {
        this.unscheduleAllCallbacks();
    },

    playEndToMainMenu: function() {
        this.removeAllObjects();
    },

    removeAllObjects: function() {
        for( var i = this.activeObjects.length; i--; ) {
            cc.pool.putInPool(this.activeObjects[i]);
        }
        this.activeObjects = [];
    },

    resetGameStart: function() {
        this.nextSpawnTime = this.getNextSpawnTime();
        this.timerSpawn = 0;
        this.curMoveDir = Math.random() < 0.5 ? -1 : 1;


        // 시작시 공 생성.
        this.schedule(this.spawnObject, 1, 3);
    },

    getNextSpawnTime: function() {
        if( g_systemManager.score < 10 ) {
            return 1;
        }
        else if( g_systemManager.score < 20 ) {
            return 0.7;
        }
        else {
            return 0.5;
        }
    },


    spawnObject: function() {
        var obj = cc.pool.getFromPool(Ball);
        if (obj.parent == null) {
            g_world.addChild(obj);
        }
        // 관리를 위해 활성객체 배열에 넣는다.
        this.activeObjects.push(obj);

        var jumpHeight = g_systemManager.randomRange(200, 540);

        if (jumpHeight > this.overGaugeHeightFromLand) {
            if (Math.random() < 0.5) {
                jumpHeight = g_systemManager.randomRange(200, this.overGaugeHeightFromLand);
            }
        }
        obj.y = jumpHeight;

        this.curMoveDir = -this.curMoveDir;
        var velX = g_systemManager.randomRangeUnit(150, 240, 20) * this.curMoveDir;

        var offsetX = velX * 1; // 1 : sideSign 이 보여질 시간(초).
        obj.x = this.curMoveDir > 0 ? -Ball.colRadius -offsetX : winSize.width + Ball.colRadius -offsetX;

        var size = g_systemManager.randomRangeUnit(1.0, 1.4, 0.2);

        obj.enable(jumpHeight, velX, size, this.seqRandBallColor.getNextElement() );

    },


    update: function(dt) {
        this.timerSpawn += dt;
        if( this.timerSpawn >= this.nextSpawnTime ) {
            this.timerSpawn = 0;
            this.nextSpawnTime = this.getNextSpawnTime();

            this.spawnObject();
        }

        // 볼 업데이트.
        for( var i = this.activeObjects.length; i--; ) {
            if( this.activeObjects[i].update(dt) ) {
                cc.pool.putInPool(this.activeObjects[i]);
                this.activeObjects.splice(i, 1);
            }
        }

    },

    updateOnPlayEnd: function(dt) {
        // 볼 업데이트.
        for( var i = this.activeObjects.length; i--; ) {
            if( this.activeObjects[i].update(dt) ) {
                cc.pool.putInPool(this.activeObjects[i]);
                this.activeObjects.splice(i, 1);
            }
        }
    }

});


var Ball = cc.Node.extend({
    ctor: function() {
        this._super();

        this.init();

        if( bShowCollider ) {
            this.drawNodeCol = new cc.DrawNode();
            this.addChild(this.drawNodeCol);
        }
    },
    init: function() {
        // 스프라이트.
        this.sprite = new cc.Sprite("#Ball0.png");
        this.addChild(this.sprite);

        // 그림자.
        this.shadow = new cc.Sprite("#Shadow.png");
        //this.addChild(this.shadow, -1);
        g_world.addChild(this.shadow, -1);
        this.shadow.y = LAND_POS_Y;


        this.normalScale = 1;
        this.sprite.scale = this.normalScale;

        // 튀기는 속성값 설정.
        // 여기서 기본값을 설정하지만, enable() 에서 다시 설정된다.
        this.jumpHeight = 640*0.75;
        this.jumpVelRelVar = 1;
        this.jumpAcc = -1000 * this.jumpVelRelVar * this.jumpVelRelVar;
        this.jumpVel = Math.sqrt(2 * Math.abs(this.jumpAcc) * this.jumpHeight);

        this.velY = 0;

        this.velX = 300;

        this.radius = Ball.colRadius;
    },
    reuse: function() {
        this.visible = true;
        this.shadow.visible = true;
        // 화면 위로 올라갔을때 위치 알림 스프라이트. (TopSign)
        if( !this.topSign ) {
            this.topSign = new cc.Sprite("#TopSign.png");
            g_guiLayer.rootSigns.addChild(this.topSign, 20);
            this.topSign.setAnchorPoint(0.5, 1);
            this.topSign.y = winSize.height;
            this.topSign.visible = false;
            this.topSign.setScaleY(1.125);
        }
        // 옆에서 나올때 위치 알림 스프라이트. (SideSign)
        if( !this.sideSign ) {
            this.sideSign = new cc.Sprite("#TopSign.png");
            g_guiLayer.rootSigns.addChild(this.sideSign, 20);
            this.sideSign.setAnchorPoint(0.5, 1);
            this.sideSign.rotation = -90;
            this.sideSign.visible = false;
        }
    },

    enable: function(jumpHeight, velX, size, spriteIndex, velRelVar) {
        this.jumpHeight = jumpHeight;
        this.jumpVelRelVar = velRelVar !== undefined ? velRelVar : 1;
        this.jumpAcc = -1000 * this.jumpVelRelVar * this.jumpVelRelVar;
        this.jumpVel = Math.sqrt(2 * Math.abs(this.jumpAcc) * this.jumpHeight);

        //this.velY = this.jumpVel;
        this.velY = 0;

        this.velX = velX;

        // 회전 속도 설정.
        this.rotateSpeed = g_systemManager.lerp(180, 360, g_systemManager.reverseLerp(150, 240, Math.abs(this.velX))) * (this.velX > 0 ? 1 : -1 );


        // 크기.
        size = size !== undefined ? size : 1;
        this.radius = Ball.colRadius * size;
        this.sprite.scale = this.normalScale * size;

        // 공 크개별 사운드 재생을 위한 인덱스 저장.
        if( size == 1 ) {
            this.soundIdx = 0;
        }
        else if( size == 1.2 ) {
            this.soundIdx = 1;
        }
        else {
            this.soundIdx = 2;
        }


        // 플레이어와의 충돌 거리.
        this.colDistSQPlayer = (g_player.COL_RADIUS_HEAD + this.radius) * (g_player.COL_RADIUS_HEAD + this.radius);

        // 사이즈에 따라 zOrder 설정.
        // 150811 size 값이 0.7~1.5 사이의 값으로 들어온다.
        // 여기에 10을 곱하고, 이값을 20에서 빼서 설정한다.
        // 13~5 사이의 값으로 설정된다.
        this.setLocalZOrder(20 - (size*10));

        // 스프라이트 설정(색상)
        this.sprite.setSpriteFrame("Ball"+spriteIndex+".png");
        this.topSign.setColor(colorsBall[spriteIndex]);
        this.sideSign.setColor(colorsBall[spriteIndex]);

        // topSign 스프라이트 스케일 설정.
        this.topSign.setScaleX(this.normalScale * size);
        // topSign 스프라이트 zOrder 설정.
        this.topSign.setLocalZOrder(30 - (size*10));

        // sideSign 스프라이트 스케일 설정.
        this.sideSign.setScaleX(this.normalScale * size);
        // sideSign 스프라이트 zOrder 설정.
        this.sideSign.setLocalZOrder(30 - (size*10));
        // sideSign 방향 설정.
        if( velX > 0 ) {
            this.sideSign.setScaleY(1);
            this.sideSign.x = 0;
        }
        else {
            this.sideSign.setScaleY(-1);
            this.sideSign.x = winSize.width;
        }

        // 그림자 위치 설정.
        this.shadow.x = this.x;

        // 콜라이더 그리기.
        if( bShowCollider ) {
            this.drawNodeCol.clear();
            this.drawNodeCol.drawCircle(cc.p(0,0), this.radius, 0, 10, false, 1, cc.color(255,0,0,255));
        }
    },


    unuse: function() {
        this.visible = false;
        this.shadow.visible = false;
        if( this.topSign ) {
            this.topSign.visible = false;
        }
        if( this.sideSign ) {
            this.sideSign.visible = false;
        }
        //this.unscheduleAllCallbacks();
    },


    update: function(dt) {
        this.x += dt * this.velX;

        if( (this.velX > 0 && this.x > winSize.width +this.radius) ||
            (this.velX < 0 && this.x < -this.radius) )
        {
            return true;
        }

        this.velY += dt * this.jumpAcc;
        this.y += dt * this.velY;

        // 땅에 닿았는가.
        if( this.y <= LAND_POS_Y +this.radius) {
            this.y = LAND_POS_Y +this.radius;
            this.velY = this.jumpVel;

            // 플레이어와 일정거리 안에 있을때 튀기는 사운드 재생.
            var dist = this.x > g_player.x ? this.x - g_player.x : g_player.x - this.x;
            if( dist < 280 && g_systemManager.systemState == SystemState.Play ) {
                //cc.audioEngine.playEffect(res.SoundBall[this.soundIdx]);
                cc.audioEngine.playEffect(res.SoundBall);
            }
        }

        // 회전.
        this.rotation += dt * this.rotateSpeed;

        // TopSign 위치 설정.
        if( this.y > GAUGE_HEIGHT + this.radius ) {
            if( !this.topSign.visible ) {
                this.topSign.visible = true;
            }
            this.topSign.x = this.x;
        }
        else {
            if( this.topSign.visible ) {
                this.topSign.visible = false;
            }
        }
        // SideSign 위치 설정.
        if( this.velX > 0 ) {
            if( this.x < -this.radius ) {
                if (!this.sideSign.visible) {
                    this.sideSign.visible = true;
                }
                this.sideSign.y = this.y;
            }
            else {
                if( this.sideSign.visible ) {
                    this.sideSign.visible = false;
                }
            }
        }
        else {
            if( this.x > winSize.width + this.radius ) {
                if (!this.sideSign.visible) {
                    this.sideSign.visible = true;
                }
                this.sideSign.y = this.y;
            }
            else {
                if( this.sideSign.visible ) {
                    this.sideSign.visible = false;
                }
            }
        }

        // 그림자 위치 설정.
        //this.shadow.y = -(this.y - LAND_POS_Y);
        this.shadow.x = this.x;
        // 그림자 스케일 설정.
        this.shadow.scale = this.sprite.scale * g_systemManager.reverseLerp(800, 0, (this.y - LAND_POS_Y) );

        return false;
    }

});
Ball.colRadius = 40;        // 사이즈 1일때(100픽셀을 기준으로 정했다.) 의 충돌 반경.
                            // 이값을 기준으로 ball.radius 가 결정된다.

var colorsBall = [
    cc.color(189,33,34),   // 빨
    cc.color(101,187,19),   // 초
    cc.color(3,154,206),   // 파
    cc.color(188,80,208),   // 보
    cc.color(239,170,3)    // 노
];
