/**
 * Created by kominpyo on 2015. 3. 4..
 */

var SwipeState = {
    Not: 0,
    Up: 1,
    Down: 2,
    Left: 3,
    Right: 4
};
var InputType = {
    Button0: 0,
    Button1: 1
};

var InputManager = cc.Class.extend({
    buttons: null,
    buttonsNum: 0,
    touchCount: 0,

    //acc: null,
    //inputH: 0,

    ctor: function() {
        this.buttons = [];
        for(var prop in InputType) {
            this.buttons[InputType[prop]] = {
                pressed: false,
                hit: false,
                flag: false,
                releaseHit: false,
                releaseFlag: false
            };

            ++this.buttonsNum;
        }

        this.touchInfo = {};
    },
    reset: function() {
        // removeAllTouchInfos
        for(var prop in this.touchInfo) {
            delete this.touchInfo[prop];
        }

        for(prop in InputType) {
            this.buttons[InputType[prop]].pressed = false;
            this.buttons[InputType[prop]].hit = false;
            this.buttons[InputType[prop]].flag = false;
            this.buttons[InputType[prop]].releaseHit = false;
            this.buttons[InputType[prop]].releaseFlag = false;
        }
    },

    touchInfo: null,
    SWIPE_MIN_LENGTH: 30,
    addTouchInfo: function(touch) {
        if (!this.touchInfo.hasOwnProperty("_" + touch.getID())) {
            var inputType;
            if( touch.getLocation().x < winCenter.x ) {
                inputType = InputType.Button0;
            }
            else {
                inputType = InputType.Button1;
            }
            this.buttons[inputType].pressed = true;

            this.touchInfo["_" + touch.getID()] = {
                inputType : inputType
                //swipeState: SwipeState.Not,
                //startPos: touch.getLocation()
            };
        }

        // 터치 입력이 5개 이상이면 모든 터치 정보를 삭제한다.
        // 터치 입력이 6개 이상이 되면 터치입력 콜백이 제대로 들어오지 않는 cocos2d-js 버그를 막기 위해 이렇게 구현한다.
        this.touchCount = 0;
        for(var prop in this.touchInfo) {
            ++this.touchCount;
        }
        if( this.touchCount >= 5 ) {
            this.reset();
        }
    },
    removeTouchInfo: function(touch) {
        if (this.touchInfo.hasOwnProperty("_" + touch.getID())) {
            var curTouchInfo = this.touchInfo["_" + touch.getID()];

            this.checkAndReleaseButton(curTouchInfo);

            delete this.touchInfo["_" + touch.getID()];
        }
    },
    checkAndReleaseButton: function(curTouchInfo) {
        // 같은 터치 입력이 없으면, 입력상태 false 로 설정.
        var countSameInput = 0;
        for( var prop in this.touchInfo ) {
            if( this.touchInfo[prop].inputType == curTouchInfo.inputType ) { ++countSameInput; }
        }
        if( countSameInput == 1 ) {
            this.buttons[curTouchInfo.inputType].pressed = false;
        }
    },

    updateTouchInfo: function(touch) {
        var touchId = "_" + touch.getID();
        if (this.touchInfo.hasOwnProperty(touchId)) {
            var curTouchInfo = this.touchInfo[touchId];
            if( curTouchInfo.inputType == InputType.Button0 && touch.getLocation().x > winCenter.x ) {

                this.checkAndReleaseButton(curTouchInfo);

                this.buttons[InputType.Button1].pressed = true;
                curTouchInfo.inputType = InputType.Button1;
            }
            else if( curTouchInfo.inputType == InputType.Button1 && touch.getLocation().x < winCenter.x ) {

                this.checkAndReleaseButton(curTouchInfo);

                this.buttons[InputType.Button0].pressed = true;
                curTouchInfo.inputType = InputType.Button0;
            }
        }
    },

    // check swipe
    updateSwipe: function(touch) {
        var touchId = "_" + touch.getID();
        if (this.touchInfo.hasOwnProperty(touchId)) {
            var curTouchInfo = this.touchInfo[touchId];
            if( curTouchInfo.swipeState == SwipeState.Not ) {
                var swipeVec = cc.pSub(touch.getLocation(), curTouchInfo.startPos);
                var swipeLength = cc.pLength(swipeVec);
                if( swipeLength > this.SWIPE_MIN_LENGTH ) {
                    curTouchInfo.swipeState = true;
                    var swipeAngle = cc.radiansToDegrees(cc.pToAngle(swipeVec));
                    if( swipeAngle > 45 && swipeAngle < 135 ) { this.upPressed = true; curTouchInfo.swipeState = SwipeState.Up; }
                    else if( swipeAngle < -45 && swipeAngle > -135 ) { this.downPressed = true; curTouchInfo.swipeState = SwipeState.Down; }
                    else if( swipeAngle > -45 && swipeAngle < 45 ) { this.rightPressed = true; curTouchInfo.swipeState = SwipeState.Right; }
                    else if( swipeAngle > 135 || swipeAngle < -135 ) { this.leftPressed = true; curTouchInfo.swipeState = SwipeState.Left; }
                }
            }
        }
    },
    update: function(dt) {
        for( var i = this.buttons.length; i--; ) {
            if (this.buttons[i].pressed) {
                if (!this.buttons[i].flag) {
                    this.buttons[i].flag = true;
                    this.buttons[i].hit = true;
                }
                else {
                    this.buttons[i].hit = false;
                }

                this.buttons[i].releaseFlag = false;
                this.buttons[i].releaseHit = false;
            }
            else {
                if (!this.buttons[i].releaseFlag) {
                    this.buttons[i].releaseFlag = true;
                    this.buttons[i].releaseHit = true;
                }
                else {
                    this.buttons[i].releaseHit = false;
                }

                this.buttons[i].flag = false;
                this.buttons[i].hit = false;
            }
        }


        // set smooth inputH
        ////this.inputH = g_systemManager.lerp(this.inputH, this.acc.x, dt * 9.4);
        //this.inputH = g_systemManager.lerp(this.inputH, this.acc.x, 0.1566);
    }
});

InputManager._sharedInstance = null;
InputManager.getInstance = function() {
    if(!this._sharedInstance) {
        this._sharedInstance = new InputManager();
    }
    return this._sharedInstance;
};
