/**
 * Created by kominpyo on 2015. 2. 23..
 */

var ColorEffectSprite = ShaderSprite.extend({
    targetColor: null,
    percent: 1,
    whiteMaxWeight: 0.5,

    ctor: function(spriteFileName, targetColor, percent, whiteMaxWeight) {
        this._super(spriteFileName || "#White4.png", res.ColorEffect_fsh);
        this.setTargetColor(targetColor !== undefined ? targetColor : {r: 0, g: 0, b: 0});
        this.setPercent(percent);
        this.setWhiteMaxWeight(whiteMaxWeight);
    },

    unuse: function() {
        this.setVisible(false);
    },
    reuse: function() {
        this.setVisible(true);
    },

    // r 이 객체이면 color4B 로 인식하고,
    // r 이 숫자이면 r, g, b 각각 0~1 사이의 값으로 인식한다.
    setTargetColor: function(r, g, b) {
        if( cc.isObject(r) ) {
            this.targetColor.x = r.r/255;
            this.targetColor.y = r.g/255;
            this.targetColor.z = r.b/255;
        }
        else if( cc.isNumber(r) ) {
            this.targetColor.x = r;
            this.targetColor.y = g;
            this.targetColor.z = b;
        }
        if ('opengl' in cc.sys.capabilities) {
            if (cc.sys.isNative) {
                this.glProgramState.setUniformVec3("u_targetColor", this.targetColor);
            }
            else {
                this.shader.use();
                this.shader.setUniformLocationWith3f(this.shader.getUniformLocationForName("u_targetColor"), this.targetColor.x, this.targetColor.y, this.targetColor.z);
                this.shader.updateUniforms();
            }
        }
    },
    setPercent: function(percent) {
        if(cc.isNumber(percent)) {
            this.percent = percent;
        }
        if ('opengl' in cc.sys.capabilities) {
            if (cc.sys.isNative) {
                this.glProgramState.setUniformFloat("u_percent", this.percent);
            }
            else {
                this.shader.use();
                this.shader.setUniformLocationWith1f(this.shader.getUniformLocationForName("u_percent"), this.percent);
                this.shader.updateUniforms();
            }
        }
    },
    setWhiteMaxWeight: function(whiteMaxWeight) {
        if(cc.isNumber(whiteMaxWeight)) {
            this.whiteMaxWeight = whiteMaxWeight;
        }
        if ('opengl' in cc.sys.capabilities) {
            if (cc.sys.isNative) {
                this.glProgramState.setUniformFloat("u_whiteMax", this.whiteMaxWeight);
            }
            else {
                this.shader.use();
                this.shader.setUniformLocationWith1f(this.shader.getUniformLocationForName("u_whiteMax"), this.whiteMaxWeight);
                this.shader.updateUniforms();
            }
        }
    },



    disableEffect: function() {
        this.unscheduleAllCallbacks();
        this.setPercent(0);
    },

    pingpongMin: 0,
    pingpongMax: 0,
    customSpeed: 0,
    pingpongTimer: 0,
    pingpongDir: 1,
    blinkPingPong: function(min, max, cycleTime, playTime) {
        this.unscheduleAllCallbacks();

        this.pingpongMin = min;
        this.pingpongMax = max;
        this.customSpeed = (max-min)*2 / cycleTime;
        this.pingpongTimer = playTime;
        this.pingpongDir = 1;
        this.setPercent(min);

        this.schedule(this.updateBlinkPingPong);
    },
    blinkPingPongOnce: function(startVal, max, playTime) {
        this.unscheduleAllCallbacks();

        if( startVal > max ) {startVal = max;}

        this.pingpongMin = 0;
        this.pingpongMax = max;
        this.customSpeed = ((max-startVal)+max)/playTime;
        this.pingpongTimer = playTime;

        this.pingpongDir = 1;
        this.setPercent(startVal);

        this.schedule(this.updateBlinkPingPong);
    },
    updateBlinkPingPong: function(dt) {
        this.percent += dt * this.pingpongDir * this.customSpeed;
        if (this.percent >= this.pingpongMax) {
            this.pingpongDir *= -1;
            this.percent = this.pingpongMax;
        }
        else if(this.percent <= this.pingpongMin) {
            this.pingpongDir *= -1;
            this.percent = this.pingpongMin;
        }

        if (this.pingpongTimer != -1) {
            this.pingpongTimer -= dt;
            if (this.pingpongTimer < 0) {
                this.percent = 0;
                this.unschedule(this.updateBlinkPingPong);
            }
        }

        this.setPercent();
    },

    blinkOnce: function(startVal, playTime) {
        this.unscheduleAllCallbacks();

        this.customSpeed = startVal / playTime;
        this.setPercent(startVal);

        this.schedule(this.updateBlinkOnce);
    },
    updateBlinkOnce: function(dt) {
        this.percent -= dt * this.customSpeed;
        if( this.percent < 0 ) {
            this.percent = 0;
            this.unschedule(this.updateBlinkOnce);
        }

        this.setPercent();
    },

    blink01: function(cycleHalfTime, playTime) {
        this.unscheduleAllCallbacks();

        this.pingpongTimer = playTime;
        this.setPercent(0);

        this.schedule(this.updateBlink01, cycleHalfTime);
    },
    updateBlink01: function(dt) {
        if( this.percent ) { this.percent = 0; }
        else { this.percent = 1; }

        if( this.pingpongTimer != -1 ) {
            this.pingpongTimer -= dt;
            if (this.pingpongTimer < 0) {
                this.percent = 0;
                this.unschedule(this.updateBlink01);
            }
        }

        this.setPercent();
    }
});



