/**
 * Created by kominpyo on 15. 5. 31..
 */
var g_effectManager;

var EffectManager = cc.Node.extend({
    ctor: function() {
        this._super();
        g_effectManager = this;

        this.setupPool();
    },

    setupPool: function() {
        // Blood
        for( var i = 80; i--; ) {
            var o = new ParticleBlood();
            cc.pool.putInPool(o);
        }
        this.activeParticleBlood = [];

        // Body Part
        for( var i = 8; i--; ) {
            var o = new BodyPart(i);
            cc.pool.putInPool(o);
        }
        this.activeBodyParts = [];
    },

    reset: function() {
        for( var i = this.activeParticleBlood.length; i--; ) {
            cc.pool.putInPool(this.activeParticleBlood[i]);
        }
        this.activeParticleBlood = [];

        for( var i = this.activeBodyParts.length; i--; ) {
            cc.pool.putInPool(this.activeBodyParts[i]);
        }
        this.activeBodyParts = [];
    },

    update: function(dt) {
        for( var i = this.activeParticleBlood.length; i--; ) {
            this.activeParticleBlood[i].update(dt);
        }
    },

    spawnBlood: function(playerPos, ballPos) {
        var vecCollide = cc.pNormalize(cc.pSub(playerPos, ballPos));
        vecCollide.x *= 400;
        vecCollide.y *= 300;

        var posX = g_player.x;
        var posY = g_player.y +24;
        for( var i = g_systemManager.randomRangeUnit(50,80,1); i--; ) { // 생성 갯수.
            var o = cc.pool.getFromPool(ParticleBlood);
            if(o.parent == null ) {
                g_world.addChild(o, 10);
            }
            this.activeParticleBlood.push(o);

            o.setPosition(posX + o.randStartPosX, posY + o.randStartPosY);

            o.velX += vecCollide.x;
            o.velY += vecCollide.y;

            //var startSpeed = g_systemManager.randomRange(200, 400);
            //var startVecNormal = cc.pNormalize(cc.p(g_systemManager.randomRange(-1,1), g_systemManager.randomRange(-0.2,1)));
            ////o.vel = cc.pMult(cc.pNormalize(cc.pSub(o.getPosition(), pos)), startSpeed);
            //o.vel = cc.pMult(startVecNormal, startSpeed);
        }

        // Body Parts
        for( var i = 8; i--; ) {
            var o = cc.pool.getFromPool(BodyPart);
            if(o.parent == null ) {
                g_world.addChild(o, 11);
            }
            this.activeBodyParts.push(o);

            o.setPosition(posX + o.randStartPosX, posY + o.randStartPosY);

            o.velX += vecCollide.x;
            o.velY += vecCollide.y;
        }

        // Effect Die
        var o = cc.pool.getFromPool(EffectDie);
        if(o.parent == null ) {
            g_world.addChild(o, 30);
        }
        o.setPosition(posX, posY+10);
    },


    spawnParticleSpread: function(pos, spriteIndex) {
        var spawnRange = 10;
        for( var i = g_systemManager.randomRangeUnit(15,20,1); i--; ) {
            var o = cc.pool.getFromPool(ParticleSpreadScaleDown);
            if(o.parent == null ) {
                g_world.addChild(o, 10);
            }
            o.scale = g_systemManager.randomRange(0.3, 0.7);
            o.setPosition(pos.x + g_systemManager.randomRange(-1, 1)*spawnRange,
                pos.y + g_systemManager.randomRange(-1, 1)*spawnRange);

            o.vel = cc.pMult(cc.pNormalize(cc.pSub(o.getPosition(), pos)), 100);
            o.setSpriteFrame("Effect"+spriteIndex+".png");
        }
    }
});

var ParticleBlood = cc.Node.extend({

    ctor: function() {
        this._super();

        this.sprite = new cc.Sprite("#Blood.png");
        this.addChild(this.sprite);

        this.velY = 0;
        this.velX = 0;
        this.randStartPosX = 0;
        this.randStartPosY = 0;

        this.accY = -2000;

        this.rotateSpeed = g_systemManager.randomRange(0.5,1)*(Math.random()<0.5? -1:1) * 2000;
    },
    reuse: function() {
        this.visible = true;

        //this.scheduleUpdate();
    },
    unuse: function() {
        this.visible = false;

        //this.unscheduleUpdate();

        // 삭제될때 랜덤 변수들을 미리 설정하여, 생성시 성능을 높인다.
        this.sprite.scale = g_systemManager.randomRange(0.5, 1);
        this.randStartPosX = g_systemManager.randomRange(-5, 5);
        this.randStartPosY = g_systemManager.randomRange(-25, 25);

        this.velY = g_systemManager.randomRange(-1150, 1150) *1.4;
        this.velX = (this.randStartPosX >= 0 ? 1 : -1) * g_systemManager.randomRange(0, 500) *1.4;
    },

    update: function(dt) {
        // 행성위에서 움직임 멈추도록 한다.
        if( this.y <= LAND_POS_Y )  {
            this.y = LAND_POS_Y;

            if( this.velY > -100 ) {
                return;
            }
            this.velY = -this.velY * 0.5;
        }

        this.x += dt * this.velX;

        this.velY += dt * this.accY;
        this.y += dt * this.velY;

        this.rotation += dt * this.rotateSpeed;

    }
});
ParticleBlood.extend = cc.Class.extend;


var BodyPart = cc.Node.extend({
    ctor: function(spriteIndex) {
        this._super();

        this.sprite = new cc.Sprite("#DeadParts"+spriteIndex+".png");
        this.addChild(this.sprite);
        //this.sprite.scale = 1.3;

        this.velY = 0;
        this.velX = 0;
        this.randStartPosX = 0;
        this.randStartPosY = 0;

        this.accY = -2000;
    },
    reuse: function() {
        this.visible = true;

        this.scheduleUpdate();
    },
    unuse: function() {
        this.visible = false;

        this.unscheduleUpdate();

        // 삭제될때 랜덤 변수들을 미리 설정하여, 생성시 성능을 높인다.
        this.randStartPosX = g_systemManager.randomRange(-5, 5);
        this.randStartPosY = g_systemManager.randomRange(-25, 25);

        this.velY = g_systemManager.randomRange(0, 900) *1.4;
        this.velX = (this.randStartPosX >= 0 ? 1 : -1) * g_systemManager.randomRange(0, 500) *1.4;

        this.rotateSpeed = g_systemManager.randomRange(0.5,1)*(Math.random()<0.5? -1:1) * 2000;

    },
    update: function(dt) {
        if( this.y <= LAND_POS_Y )  {
            this.y = LAND_POS_Y;

            if( this.velY > -100 ) {
                this.unscheduleUpdate();
                return;
            }
            this.velY = -this.velY * 0.7;
        }

        this.x += dt * this.velX;

        this.velY += dt * this.accY;
        this.y += dt * this.velY;

        this.rotation += dt * this.rotateSpeed;

    }
});

//var EffectDie = EffectSpineOnce.extend({
//    ctor: function() {
//        this._super(res.EffectDead_json, res.EffectDead_atlas);
//    }
//});
var EffectDie = EffectActionOnce.extend({
    ctor: function() {
        var act = g_systemManager.createAnimate(0.033, "EffectBlood");
        this._super(act);
    }
});


var ParticleSpreadScaleDown = cc.Sprite.extend({
    vel: null,
    scaleSpeed: -1.5,

    ctor: function() {
        this._super();
        this.vel = cc.p(0,0);
    },
    reuse: function() {
        this.visible = true;

        this.scheduleUpdate();
    },
    unuse: function() {
        this.visible = false;

        this.unscheduleUpdate();
    },

    update: function(dt) {
        this.scale += dt * this.scaleSpeed;
        if( this.scale <= 0 ) {
            cc.pool.putInPool(this);
            return;
        }

        this.setPosition(cc.pAdd(this.getPosition(), cc.pMult(this.vel, dt)));

    }
});