/**
 * Created by kominpyo on 15. 7. 8..
 */

var LightSprite = GLNodeBase.extend({
    lightHalfLengthInPixels: 128,

    ctor: function(imageFileName) {
        this._super(imageFileName || "White4.png", res.lightShader_fsh, res.lightShader_vsh);

        this.setTexMaskLight();
        this.setupUniformLocations();
    },

    drawFunc: function() {

        this.shader.use();
        this.shader.setUniformsForBuiltins();

        //cc.glBlendFunc(this.getBlendFunc().src, this.getBlendFunc().dst);
        //cc.glBlendFunc(cc.SRC_ALPHA, cc.ONE_MINUS_SRC_ALPHA);

        gl.bindTexture(gl.TEXTURE_2D, this.my_texture.getName());
        //gl.bindTexture(gl.TEXTURE_2D, this.texMaskLight.getName());

        cc.glEnableVertexAttribs( cc.VERTEX_ATTRIB_FLAG_POSITION | cc.VERTEX_ATTRIB_FLAG_COLOR | cc.VERTEX_ATTRIB_FLAG_TEX_COORDS );


        // Draw fullscreen Square
        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexPositionBuffer);
        gl.vertexAttribPointer(cc.VERTEX_ATTRIB_POSITION, 2, gl.FLOAT, false, 0, 0);

        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexTextureBuffer);
        gl.vertexAttribPointer(cc.VERTEX_ATTRIB_TEX_COORDS, 2, gl.FLOAT, false, 0, 0);

        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexColorBuffer);
        gl.vertexAttribPointer(cc.VERTEX_ATTRIB_COLOR, 4, gl.FLOAT, true, 0, 0);

        // active maskLight texture
        gl.activeTexture(gl.TEXTURE1);
        gl.bindTexture(gl.TEXTURE_2D, this.texMaskLight.getName());


        gl.drawArrays(gl.TRIANGLE_STRIP, 0, 4);

        gl.bindTexture(gl.TEXTURE_2D, null);
        gl.bindBuffer(gl.ARRAY_BUFFER, null);

        gl.activeTexture(gl.TEXTURE0);
    },

    setTexMaskLight: function() {
        var texture2d = this.texMaskLight = cc.textureCache.addImage(res.MaskLight_png);
        gl.bindTexture(gl.TEXTURE_2D, texture2d.getName());

        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
        gl.bindTexture(gl.TEXTURE_2D, null);

        var program = this.shader.getProgram();
        if( this.ulMaskLight === undefined ) {
            this.ulMaskLight = gl.getUniformLocation( program, "u_mask");
        }
        this.shader.setUniformLocationI32( this.ulMaskLight, 1);

    },

    setupUniformLocations: function() {

        var program = this.shader.getProgram();

        // set uniform location of light mask texture half length
        this.lightHalfLengthRatio = cc.p(this.lightHalfLengthInPixels / this.glnode.width, this.lightHalfLengthInPixels / this.glnode.height);
        if( this.ulLightHalfLength === undefined ) {
            this.ulLightHalfLength = gl.getUniformLocation( program, "u_lightHalfLength");
        }
        this.shader.setUniformLocationF32( this.ulLightHalfLength, this.lightHalfLengthRatio.x, this.lightHalfLengthRatio.y);

        // set uniform location of light ambient
        this.ambient = 0.2;
        this.ulAmbient = gl.getUniformLocation( program, "u_ambient");
        this.shader.setUniformLocationF32( this.ulAmbient, this.ambient);

        // set uniform location of light intensity
        this.lightIntensity = 1;
        this.ulLightIntensity = gl.getUniformLocation( program, "u_lightIntensity");
        this.shader.setUniformLocationF32( this.ulLightIntensity, this.lightIntensity);

        // set uniform location of light positions
        this.posLights = new Array(LightSprite.MAX_LIGHTS_NUM);
        this.ulPosLights = new Array(LightSprite.MAX_LIGHTS_NUM);

        for(var i = LightSprite.MAX_LIGHTS_NUM; i--; ) {
            this.posLights[i] = cc.p(-1000, -1000);
            this.ulPosLights[i] = gl.getUniformLocation(program, "u_lightPosition"+i);
            this.shader.setUniformLocationF32(this.ulPosLights[i], this.posLights[i].x, this.posLights[i].y);
        }

        // test uniform array
        //this.ulArrayPosLight = gl.getUniformLocation(program, "u_lightPosition");
        //this.shader.setUniformLocationWith2fv(this.ulArrayPosLight, this.posLights);

        // set uniform location of texture length
        //this.lightHalfLengthRatio = cc.p(this.lightHalfLengthInPixels / this.glnode.width, this.lightHalfLengthInPixels / this.glnode.height);
        //if( this.ulLightHalfLength === undefined ) {
        //    this.ulLightHalfLength = gl.getUniformLocation( program, "u_lightHalfLength");
        //}
        //this.shader.setUniformLocationF32( this.ulLightHalfLength, this.lightHalfLengthRatio.x, this.lightHalfLengthRatio.y);

    },

/*
    setLightPos: function(worldPosLight) {
        // 좌표변환 후에 정점셰이더 유니폼 변수에 넣어준다.
        // 월드 좌표를 이 스프라이트의 상대위치 비율값으로 변환한다.
        // 월드 좌표계는 좌하단이 원점이고, 텍스처 좌표게는 좌상단이 원점이므로 이부분에서의 좌표변환을 하고,
        // 텍스처 길이 대비 값을 계산하는 과정이 포함된다.
        this.posLight = this.convertToNodeSpaceAR(worldPosLight);
        //cc.log("texSize: "+this.glnode.width+", "+this.glnode.height);
        //cc.log(this.posLight.x+", "+this.posLight.y);

        this.posLight.x = (this.posLight.x+this.glnode.width*0.5) / this.glnode.width;
        this.posLight.y = (this.glnode.height - (this.posLight.y+this.glnode.height*0.5)) / this.glnode.height;
        //cc.log(this.posLight.x+", "+this.posLight.y);

        // set uniform location of light position
        this.shader.use();
        this.shader.setUniformLocationF32(this.ulLightPos, this.posLight.x, this.posLight.y);
        this.shader.updateUniforms();
    },
*/

    posLights: null,
    setPosLights: function(index, worldPosLight) {
        if( index >= LightSprite.MAX_LIGHTS_NUM ) {
            return;
        }

        // 좌표변환 후에 정점셰이더 유니폼 변수에 넣어준다.
        // 월드 좌표를 이 스프라이트의 상대위치 비율값으로 변환한다.
        // 월드 좌표계는 좌하단이 원점이고, 텍스처 좌표게는 좌상단이 원점이므로 이부분에서의 좌표변환을 하고,
        // 텍스처 길이 대비 값을 계산하는 과정이 포함된다.
        this.posLights[index] = this.convertToNodeSpaceAR(worldPosLight);
        //cc.log("texSize: "+this.glnode.width+", "+this.glnode.height);
        //cc.log(this.posLight.x+", "+this.posLight.y);

        this.posLights[index].x = (this.posLights[index].x+this.glnode.width*0.5) / this.glnode.width;
        this.posLights[index].y = (this.glnode.height - (this.posLights[index].y+this.glnode.height*0.5)) / this.glnode.height;
        //cc.log(this.posLight.x+", "+this.posLight.y);

        // set uniform location of light position
        this.shader.use();
        this.shader.setUniformLocationF32(this.ulPosLights[index], this.posLights[index].x, this.posLights[index].y);
        this.shader.updateUniforms();

    },

    setLightHalfLength: function(halfLengthInPixels) {
        this.lightHalfLengthInPixels = halfLengthInPixels;
        this.lightHalfLengthRatio.x = this.lightHalfLengthInPixels / this.glnode.width;
        this.lightHalfLengthRatio.y = this.lightHalfLengthInPixels / this.glnode.height;

        this.shader.use();
        this.shader.setUniformLocationF32( this.ulLightHalfLength, this.lightHalfLengthRatio.x, this.lightHalfLengthRatio.y);
        this.shader.updateUniforms();
    },

    setAmbient: function(ambient) {
        this.ambient = ambient;

        this.shader.use();
        this.shader.setUniformLocationF32( this.ulAmbient, ambient);
        this.shader.updateUniforms();
    },

    setLightIntensity: function(lightIntensity) {
        this.lightIntensity = lightIntensity;

        this.shader.use();
        this.shader.setUniformLocationF32( this.ulLightIntensity, lightIntensity);
        this.shader.updateUniforms();
    }

});
LightSprite.MAX_LIGHTS_NUM = 1;
