/**
 * Created by kominpyo on 15. 7. 24..
 */

var PButtonState = {
    Waiting: 0,
    Tracking: 1
};

var PButton = cc.Node.extend({
    state: PButtonState.Waiting,

    normalSprite: null,
    onClicked: null,
    onSelectedAction: null,
    onUnselectedAction: null,
    bIndividual: false,


    onExit: function() {
        if( this.onSelectedAction !== null ) { this.onSelectedAction.release(); }
        if( this.onUnselectedAction !== null ) { this.onUnselectedAction.release(); }
        if( this.normalSpriteFrame !== null ) { this.normalSpriteFrame.release(); }

        this._super();
    },

    //ctor: function(normalSprite, onClickedFunc, target, onSelectedAction, onUnselectedAction, bIndividual) {
    ctor: function(normalSprite, selectedSpriteFrameName, onClickedFunc, target, onSelectedAction, onUnselectedAction, bIndividual) {
        this._super();

        this.normalSprite = normalSprite;
        this.selectedSpriteFrameName = selectedSpriteFrameName !== undefined ? selectedSpriteFrameName : null;
        this.onClicked = cc.isFunction(onClickedFunc) ? onClickedFunc : function(){};
        this.target = target !== undefined ? target : null;
        this.onSelectedAction = onSelectedAction !== undefined ? onSelectedAction : null;
        this.onUnselectedAction = onUnselectedAction !== undefined ? onUnselectedAction : null;
        this.bIndividual = bIndividual !== undefined ? bIndividual : false;

        this.addChild(normalSprite);
        if( this.selectedSpriteFrameName !== null ) {
            this.normalSpriteFrame = this.normalSprite.getSpriteFrame();
            if( cc.sys.isNative ) {
                this.normalSpriteFrame.retain();
            }
        }

        if( onSelectedAction !== null ) {
            onSelectedAction.retain();
        }
        if( onUnselectedAction !== null ) {
            onUnselectedAction.retain();
        }

        cc.eventManager.addListener({
            event: cc.EventListener.TOUCH_ONE_BY_ONE,
            swallowTouches: true,
            onTouchBegan: function (touch, event) {
                var target = event.getCurrentTarget();
                if( target.state !== PButtonState.Waiting || !target.visible || !PButton.enabled ) {
                    return false;
                }
                for (var c = target.parent; c != null; c = c.parent) {
                    if (!c.isVisible())
                        return false;
                }
                if( !target.bIndividual && PButton.bPressedOne ) {
                    return true;
                }

                var touchLocation = touch.getLocation();

                var localPos = target.normalSprite.convertToNodeSpace(touchLocation);
                var r = target.rect();
                r.x = 0;
                r.y = 0;
                if (cc.rectContainsPoint(r, localPos)) {
                    target.state = PButtonState.Tracking;
                    target.selected();

                    if( !target.bIndividual ) {
                        PButton.bPressedOne = true;
                    }

                    return true;
                }

                return false;
            },
            onTouchMoved: function (touch, event) {
                var target = event.getCurrentTarget();
                if( target.state !== PButtonState.Tracking ) {
                    return;
                }

                var touchLocation = touch.getLocation();

                var localPos = target.normalSprite.convertToNodeSpace(touchLocation);
                var r = target.rect();
                r.x = 0;
                r.y = 0;
                if( target.bSelected && !cc.rectContainsPoint(r, localPos) ) {
                    target.unselected();
                }
                else if( !target.bSelected && cc.rectContainsPoint(r, localPos) ) {
                    target.selected();
                }
            },
            onTouchEnded: function (touch, event) {
                var target = event.getCurrentTarget();
                if( target.state !== PButtonState.Tracking ) {
                    return;
                }

                if( target.bSelected ) {
                    target.unselected();
                    //target.onClicked();
                    target.activate();
                }
                target.state = PButtonState.Waiting;

                if( !target.bIndividual ) {
                    PButton.bPressedOne = false;
                }
            },
            onTouchCancelled: function (touch, event) {
                var target = event.getCurrentTarget();
                target.onTouchEnded(touch, event)
            }
        }, this);

    },

    rect: function() {
        return this.normalSprite.getContentSize();
    },

    bSelected: false,
    selected: function() {
        this.bSelected = true;

        if( this.onSelectedAction !== null ) {
            this.normalSprite.stopAllActions();
            this.normalSprite.runAction(this.onSelectedAction);
        }

        if( this.selectedSpriteFrameName !== null ) {
            this.normalSprite.setSpriteFrame(this.selectedSpriteFrameName);
        }
    },

    unselected: function() {
        this.bSelected = false;

        if( this.onUnselectedAction !== null ) {
            this.normalSprite.stopAllActions();
            this.normalSprite.runAction(this.onUnselectedAction);
        }

        if( this.selectedSpriteFrameName !== null ) {
            this.normalSprite.setSpriteFrame(this.normalSpriteFrame);
        }
    },

    activate: function() {
        var locTarget = this.target, locCallback = this.onClicked;
        if (!locCallback)
            return;
        if (locTarget && cc.isString(locCallback)) {
            locTarget[locCallback](this);
        } else if (locTarget && cc.isFunction(locCallback)) {
            locCallback.call(locTarget, this);
        } else
            locCallback(this);
    }
});
PButton.bPressedOne = false;
PButton.enabled = true;
