/**
 * Created by kominpyo on 2015. 3. 17..
 */

function SequencedRandom(elementsNum) {
    this.elementsNum = elementsNum >= 1 ? elementsNum : 1;       // element 갯수설정. 잘못된 값이면 1로 설정.
    this.lastElement = Math.floor(Math.random()*elementsNum);   // 마지막에 뽑은 element. 최초 랜덤하게 설정한다.
    this.elements = [];                                         // element 배열 초기화.
    this.reset();                                               // 초기화.
}

// 객체 초기화 함수.
// 다시 처음부터 인덱스를 뽑고자 할때 이 함수를 호출한다.
SequencedRandom.prototype.reset = function(elementsNum, lastElement) {
    if( elementsNum && lastElement) {       // 매개변수를 받은 경우, 프로퍼티에 그대로 대입한다. 그리고 재귀 호출한다.
        this.elementsNum = elementsNum;
        this.lastElement = lastElement;
        arguments.callee();
    }
    else {
        // elements 배열에 인덱스들을 차례로 넣어준다.
        // 단, lastElement 는 마지막에 넣어준다.
        // 이유는 이 객체를 리셋한 후 첫 인덱스를 뽑을때, 마지막으로 뽑은 인덱스를 뽑지 않기 위함이다.
        this.elements = [];
        for(var i = this.elementsNum; i--; ) {
            if( i != this.lastElement ) {
                this.elements.push(i);
            }
        }
        this.elements.push(this.lastElement);
    }
};

// 실제 랜덤 인덱스를 뽑는 함수. 반복되지 않는 랜덤값을 반환한다.
SequencedRandom.prototype.getNextElement = function() {
    var randIndex = 0, randElement = 0;
    if( this.elements.length == 0 ) {
        // error!!
        return -1;
    }
    // elements 배열에 인덱스가 1개 남은 경우, 이값을 반환하고 초기화.
    else if( this.elements.length == 1 ) {
        randElement = this.elements[0];
        this.lastElement = randElement;

        this.reset();
    }
    // elements 배열이 최대 갯수일 경우. 즉, 리셋후 첫 인덱스인 경우이다.
    // elements 배열 마지막에 이전에 뽑은 lastElement 가 들어있다.
    // 이를 제외하고 랜덤한 값을 뽑아서 반환한다.
    else if( this.elements.length == this.elementsNum ) {
        randIndex = Math.floor(Math.random()*(this.elementsNum-1));
        randElement = this.elements[randIndex];
        this.elements.splice(randIndex, 1);
    }
    // 보통의 경우. elements 배열중 랜덤값을 뽑아 반환한다.
    else {
        randIndex = Math.floor(Math.random()*this.elements.length);
        randElement = this.elements[randIndex];
        this.elements.splice(randIndex, 1);
    }

    return randElement;
};

// elements 배열에서 특정 인덱스값을 일부러 빼내고자 할때 이 함수를 사용한다.
SequencedRandom.prototype.getElement = function(element) {
    for( var i = this.elements.length; i--; ) {
        if( this.elements[i] == element ) {
            var ret = this.elements[i];
            this.elements.splice(i, 1);
            if( this.elements.length == 0 ) {
                this.reset();
            }
            return ret;
        }
    }

    return -1;
};
