/**
 * Created by kominpyo on 15. 7. 19..
 */


var Trail = cc.Node.extend({
    ctor: function(followObj, startHalfWidth, pointsNum, textureName, color) {
        this._super();

        this.pointsNum = pointsNum > 2 ? pointsNum : 2;
        this.followObj = followObj;

        this.startHalfWidth = startHalfWidth;
        this.endHalfWidth = 0;

        this.colorTrail = color !== undefined ? color : {r: 1, g: 1, b:1 };
        this.color = cc.color(this.colorTrail.r*255, this.colorTrail.g*255, this.colorTrail.b*255);
        this.curColor = cc.color(this.getColor());
        this.lastColor = cc.color(this.getColor());

        if( cc.isString(textureName) ) {
            this.texId = cc.textureCache.addImage(textureName).getName();
            this.tex = cc.textureCache.addImage(textureName);
        }
        else {
            this.texId = null;
        }


        if( 'opengl' in cc.sys.capabilities ) {

            var glnode = new cc.GLNode();
            this.addChild(glnode);
            this.glnode = glnode;

            // 현재 내가 구현한 트레일은 cocos2d-js sample project를 참고했다.
            // 그런데 이 GLNode 의 zOrder 앞, 뒤로 같은 텍스처가 사용되면,
            // 뒤에 그려질 텍스처가 제대로 나오지 않고, 검게 나오는 문제가 발생한다.
            // 그래서 텍스처를 일부러 한번 바꾸기 위해 아래와 같은 코드를 첨부한다.
            // 이렇게 하면 문제가 없어진다.
            this.spriteTemp = new cc.Sprite("#White4A.png");
            this.glnode.addChild(this.spriteTemp, 1);
            this.spriteTemp.setColor(cc.color(0,0,0));
            this.spriteTemp.setOpacity(1);
            //this.spriteTemp.setPosition(winCenter);

            this.initVertexBuffer3();
            //this.initColorBuffer();
            this.initColorBuffer1();

            // 텍스처가 없는 경우의 메서드 정의.
            if( this.texId == null ) {

                this.shader = cc.shaderCache.getProgram("ShaderPositionColor");


                glnode.draw = function() {
                    this.updateVertexBuffer1();

                    this.shader.use();
                    this.shader.setUniformsForBuiltins();

                    cc.glEnableVertexAttribs( cc.VERTEX_ATTRIB_FLAG_COLOR | cc.VERTEX_ATTRIB_FLAG_POSITION);

                    gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexPositionBuffer);
                    gl.vertexAttribPointer(cc.VERTEX_ATTRIB_POSITION, 2, gl.FLOAT, false, 0, 0);

                    gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexColorBuffer);
                    gl.vertexAttribPointer(cc.VERTEX_ATTRIB_COLOR, 4, gl.FLOAT, true, 0, 0);

                    gl.drawArrays(gl.TRIANGLE_STRIP, 0, this.pointsNum*2);

                    gl.bindTexture(gl.TEXTURE_2D, null);
                    gl.bindBuffer(gl.ARRAY_BUFFER, null);

                }.bind(this);

                this.updateVertexBuffer1();
            }
            // 텍스처가 있는 경우의 메서드 정의.
            else {

                this.shader = cc.shaderCache.getProgram("ShaderPositionTextureColor");
                //this.shader = cc.shaderCache.getProgram("ShaderPositionTexture");

                this.initTextureBuffer();

                glnode.draw = function() {
                    this.updateVertexBuffer3();
                    this.updateColorBuffer1();

                    this.shader.use();
                    this.shader.setUniformsForBuiltins();

                    gl.bindTexture(gl.TEXTURE_2D, this.texId);

                    cc.glEnableVertexAttribs( cc.VERTEX_ATTRIB_FLAG_POSITION | cc.VERTEX_ATTRIB_FLAG_COLOR | cc.VERTEX_ATTRIB_FLAG_TEX_COORDS );
                    //cc.glEnableVertexAttribs( cc.VERTEX_ATTRIB_FLAG_POSITION | cc.VERTEX_ATTRIB_FLAG_TEX_COORDS );

                    gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexPositionBuffer);
                    gl.vertexAttribPointer(cc.VERTEX_ATTRIB_POSITION, 2, gl.FLOAT, false, 0, 0);

                    gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexTextureBuffer);
                    gl.vertexAttribPointer(cc.VERTEX_ATTRIB_TEX_COORDS, 2, gl.FLOAT, false, 0, 0);

                    gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexColorBuffer);
                    gl.vertexAttribPointer(cc.VERTEX_ATTRIB_COLOR, 4, gl.FLOAT, true, 0, 0);

                    gl.drawArrays(gl.TRIANGLE_STRIP, 0, this.pointsNum*2);

                    gl.bindTexture(gl.TEXTURE_2D, null);
                    gl.bindBuffer(gl.ARRAY_BUFFER, null);

                }.bind(this);

                this.updateVertexBuffer3();
            }


        }
    },

    // 트레일 정점 버퍼 초기화.
    // 풀링된 오브젝트에 트레일을 붙여 사용할때, 생성시 이 함수를 호출해줘야한다.
    resetVertexBuffer: function() {
        //this.curPos = this.followObj.getPosition();
        //this.curPos = this.followObj.convertToWorldSpace(cc.p(0,0));
        this.curPos = g_world.convertToNodeSpace(this.followObj.convertToWorldSpace(cc.p(0,0)));
        this.lastPos = this.curPos;

        for( var i = this.pointsNum; i--; )
        {
            var j = i*4;
            this.f32arrayVertices[j] = this.curPos.x;
            this.f32arrayVertices[j+1] = this.curPos.y;
            this.f32arrayVertices[j+2] = this.curPos.x;
            this.f32arrayVertices[j+3] = this.curPos.y;
        }


        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexPositionBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayVertices, gl.STATIC_DRAW);

        gl.bindBuffer(gl.ARRAY_BUFFER, null);

        this.spriteTemp.setPosition(g_world.convertToNodeSpace(this.followObj.convertToWorldSpace(cc.p(0,0))));
    },

    addPoints: function(numPointAdd) {
        // reset Vertex Buffer
        this.pointsNum += numPointAdd;
        var newf32arrayVertices = new Float32Array(this.pointsNum*4);

        var arrayAdd = new Array(numPointAdd*4);
        var curLengthVertexBuffer = this.f32arrayVertices.length;
        var x1 = this.f32arrayVertices[curLengthVertexBuffer-4];
        var y1 = this.f32arrayVertices[curLengthVertexBuffer-3];
        var x2 = this.f32arrayVertices[curLengthVertexBuffer-2];
        var y2 = this.f32arrayVertices[curLengthVertexBuffer-1];
        for( var i = numPointAdd; i--; ) {
            var j = i*4;
            arrayAdd[j] = x1;
            arrayAdd[j+1] = y1;
            arrayAdd[j+2] = x2;
            arrayAdd[j+3] = y2;
        }
        newf32arrayVertices.set(this.f32arrayVertices, 0);
        newf32arrayVertices.set(arrayAdd, curLengthVertexBuffer);

        this.f32arrayVertices = newf32arrayVertices;
        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexPositionBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayVertices, gl.STATIC_DRAW);
        gl.bindBuffer(gl.ARRAY_BUFFER, null);


        // reset Color Buffer
        var newf32arrayColors = new Float32Array(this.pointsNum*8);

        var arrayAddColor = new Array(numPointAdd*8);
        var curLengthColorBuffer = this.f32arrayColors.length;
        for( var i = numPointAdd*2; i--; )
        {
            var j = i*4;
            arrayAddColor[j] = this.colorTrail.r;
            arrayAddColor[j+1] = this.colorTrail.g;
            arrayAddColor[j+2] = this.colorTrail.b;
            arrayAddColor[j+3] = 1;
        }
        newf32arrayColors.set(this.f32arrayColors, 0);
        newf32arrayColors.set(arrayAddColor, curLengthColorBuffer);

        this.f32arrayColors = newf32arrayColors;
        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexColorBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayColors, gl.STATIC_DRAW);
        gl.bindBuffer(gl.ARRAY_BUFFER, null);


        // reset Texture Buffer
        var texCoords = new Array(this.pointsNum);
        var fLerpT = 0;
        for( var i = this.pointsNum; i--; ) {
            fLerpT = ((this.pointsNum-1)-i)/(this.pointsNum-1);
            var j = i*4;
            texCoords[j] = fLerpT;
            texCoords[j+1] = 0;
            texCoords[j+2] = fLerpT;
            texCoords[j+3] = 1;
        }
        this.f32arrayTexCoords = new Float32Array(texCoords);

        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexTextureBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayTexCoords, gl.STATIC_DRAW);

        gl.bindBuffer(gl.ARRAY_BUFFER, null);
    },

    resetPoints: function(pointsNum) {
        this.pointsNum = pointsNum;

        this.initTextureBuffer();
        this.initColorBuffer1();
        this.initVertexBuffer3();
    },

    initTextureBuffer: function() {
        // Init Texture
        gl.bindTexture(gl.TEXTURE_2D, this.texId);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
        gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
        gl.bindTexture(gl.TEXTURE_2D, null);

        var indexT = 0;
        var texCoords = [];
        var fLerpT = 0;
        for( var i = 0; i < this.pointsNum; ++i ) {
            fLerpT = ((this.pointsNum-1)-i)/(this.pointsNum-1);
            // set texCoords
            texCoords[indexT++] = fLerpT;
            texCoords[indexT++] = 0;
            texCoords[indexT++] = fLerpT;
            texCoords[indexT++] = 1;
        }

        this.f32arrayTexCoords = new Float32Array(texCoords);

        var squareVertexTextureBuffer = this.squareVertexTextureBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, squareVertexTextureBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayTexCoords, gl.STATIC_DRAW);

        gl.bindBuffer(gl.ARRAY_BUFFER, null);
    },

    initColorBuffer: function() {
        var indexC = 0;
        var colors = [];
        //var fLerpT = 0;
        var fLerpT = 1;
        for( var i = 0; i < this.pointsNum; ++i ) {
            //fLerpT = ((this.pointsNum-1)-i)/(this.pointsNum-1);
            // set color
            if( cc.sys.isNative ) {
                colors[indexC++] = this.colorTrail.r * fLerpT;
                colors[indexC++] = this.colorTrail.g * fLerpT;
                colors[indexC++] = this.colorTrail.b * fLerpT;
                colors[indexC++] = fLerpT;

                colors[indexC++] = this.colorTrail.r * fLerpT;
                colors[indexC++] = this.colorTrail.g * fLerpT;
                colors[indexC++] = this.colorTrail.b * fLerpT;
                colors[indexC++] = fLerpT;
            }
            else {
                colors[indexC++] = this.colorTrail.r;
                colors[indexC++] = this.colorTrail.g;
                colors[indexC++] = this.colorTrail.b;
                colors[indexC++] = fLerpT;

                colors[indexC++] = this.colorTrail.r;
                colors[indexC++] = this.colorTrail.g;
                colors[indexC++] = this.colorTrail.b;
                colors[indexC++] = fLerpT;
            }
        }

        var squareVertexColorBuffer = this.squareVertexColorBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, squareVertexColorBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(colors), gl.STATIC_DRAW);

        gl.bindBuffer(gl.ARRAY_BUFFER, null);


    },
    initColorBuffer1: function() {
        this.f32arrayColors = new Float32Array(this.pointsNum*8);

        for( var i = this.pointsNum*2; i--; )
        {
            var j = i*4;
            this.f32arrayColors[j] = this.colorTrail.r;
            this.f32arrayColors[j+1] = this.colorTrail.g;
            this.f32arrayColors[j+2] = this.colorTrail.b;
            this.f32arrayColors[j+3] = 1;
        }


        this.squareVertexColorBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexColorBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayColors, gl.STATIC_DRAW);

        gl.bindBuffer(gl.ARRAY_BUFFER, null);

    },
    updateColorBuffer1: function() {
        this.curColor = this.getColor();
        if( this.curColor.r != this.lastColor.r ||
            this.curColor.g != this.lastColor.g ||
            this.curColor.b != this.lastColor.b )
        {
            this.lastColor.r = this.curColor.r;
            this.lastColor.g = this.curColor.g;
            this.lastColor.b = this.curColor.b;

            this.colorTrail.r = this.curColor.r / 255;
            this.colorTrail.g = this.curColor.g / 255;
            this.colorTrail.b = this.curColor.b / 255;
            for (var i = this.pointsNum * 2; i--;) {
                var j = i * 4;
                this.f32arrayColors[j] = this.colorTrail.r;
                this.f32arrayColors[j+1] = this.colorTrail.g;
                this.f32arrayColors[j+2] = this.colorTrail.b;
            }


            gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexColorBuffer);
            gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayColors, gl.STATIC_DRAW);

            gl.bindBuffer(gl.ARRAY_BUFFER, null);
        }
    },


    // init 과 update 가 짝을 이룬다. 뒤에 번호가 짝을 이루는 번호다.
    // 각각 구현방식이 다르다. 3으로 갈수록 퍼포먼스가 높다.
    // 1번 구현방식 : 이동한 정점을 기록하고, 수직벡터를 구해서 양옆의 정점을 구한다.
    // 2번 구현방식 : 정점이 매프레임마다 이동할 거리를 미리 계산해 두고, 이값을 증감해준다.
    // 3번 구현방식 : 정점이 수정을 최소화한 방식이다. 정점으로 트레일의 모양을 정하지 않고 텍스처로 대신한다.
    //             그리고 정점은 이전정점을 그대로 다음 위치로 넘기는 방식이다.

    initVertexBuffer1: function() {
        this.pts = [];
        for( var i = this.pointsNum; i--; ) {
            this.pts[i] = this.followObj.getPosition();
        }


        this.f32arrayVertices = new Float32Array(this.pointsNum*4);
        var indexVf = 0;
        this.vecVerticals = [];
        var fLerpT = 0;
        this.halfWidths = [];
        var vecDir;
        for( var i = 0; i < this.pointsNum; ++i ) {
            // set vertex
            if( i != this.pointsNum-1 ) {
                vecDir = cc.p(this.pts[i].x-this.pts[i+1].x, this.pts[i].y-this.pts[i+1].y);
                vecDir = cc.pNormalize(vecDir);
                this.vecVerticals[i] = cc.p(vecDir.y, -vecDir.x);
            }
            // 마지막 점의 수직벡터는 이전 지점의 수직벡터를 사용.
            else {
                this.vecVerticals[i] = this.vecVerticals[i-1];
            }

            fLerpT = ((this.pointsNum-1)-i)/(this.pointsNum-1);
            this.halfWidths[i] = this.endHalfWidth + (this.startHalfWidth-this.endHalfWidth)*fLerpT;


            this.f32arrayVertices[indexVf++] = this.pts[i].x + this.vecVerticals[i].x * this.halfWidths[i];
            this.f32arrayVertices[indexVf++] = this.pts[i].y + this.vecVerticals[i].y * this.halfWidths[i];
            this.f32arrayVertices[indexVf++] = this.pts[i].x - this.vecVerticals[i].x * this.halfWidths[i];
            this.f32arrayVertices[indexVf++] = this.pts[i].y - this.vecVerticals[i].y * this.halfWidths[i];

        }


        this.squareVertexPositionBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexPositionBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayVertices, gl.STATIC_DRAW);

        gl.bindBuffer(gl.ARRAY_BUFFER, null);
    },

    initVertexBuffer2: function() {
        this.f32arrayVertices = new Float32Array(this.pointsNum*4);

        this.curPos = this.followObj.getPosition();
        this.lastPos = this.followObj.getPosition();
        this.deltaHalfWidths = new Array(this.pointsNum);
        this.deltaHalfWidthMax = (this.startHalfWidth-this.endHalfWidth) / this.pointsNum;

        for( var i = this.pointsNum; i--; )
        {
            var j = i*4;
            this.f32arrayVertices[j] = this.curPos.x;
            this.f32arrayVertices[j+1] = this.curPos.y;
            this.f32arrayVertices[j+2] = this.curPos.x;
            this.f32arrayVertices[j+3] = this.curPos.y;

            this.deltaHalfWidths[i] = cc.p(0, 0);
        }

        this.squareVertexPositionBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexPositionBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayVertices, gl.STATIC_DRAW);

        gl.bindBuffer(gl.ARRAY_BUFFER, null);
    },

    initVertexBuffer3: function() {
        //this.curPos = this.followObj.getPosition();
        //this.curPos = this.followObj.convertToWorldSpace(cc.p(0,0));
        this.curPos = g_world.convertToNodeSpace(this.followObj.convertToWorldSpace(cc.p(0,0)));
        this.lastPos = this.curPos;

        this.f32arrayVertices = new Float32Array(this.pointsNum*4);

        for( var i = this.pointsNum; i--; )
        {
            var j = i*4;
            this.f32arrayVertices[j] = this.curPos.x;
            this.f32arrayVertices[j+1] = this.curPos.y;
            this.f32arrayVertices[j+2] = this.curPos.x;
            this.f32arrayVertices[j+3] = this.curPos.y;
        }


        this.squareVertexPositionBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexPositionBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayVertices, gl.STATIC_DRAW);

        gl.bindBuffer(gl.ARRAY_BUFFER, null);

    },


    // 1번 구현방식 : 이동한 정점을 기록하고, 수직벡터를 구해서 양옆의 정점을 구한다.
    updateVertexBuffer1: function() {
        // 첫번째 구현.
        this.pts.pop();
        this.pts.unshift(this.followObj.getPosition());

        // 수직벡터 업데이트.
        // 마지막 벡터는 버리고, 새 벡터를 배열 앞에 추가한다.
        // 새로운 마지막 벡터를 이전 벡터와 같은 값을 준다. (마지막 방향벡터가 이전 point 를 향하도록 하기 위함.)
        this.vecVerticals.pop();

        var vecDir = cc.pNormalize(cc.p(this.pts[0].x-this.pts[1].x, this.pts[0].y-this.pts[1].y));
        this.vecVerticals.unshift(cc.p(vecDir.y, -vecDir.x));

        this.vecVerticals[this.pointsNum-1] = this.vecVerticals[this.pointsNum-2];


        for( var i = this.pointsNum; i--; )
        {
            var j = i*4;
            this.f32arrayVertices[j] = this.pts[i].x + this.vecVerticals[i].x * this.halfWidths[i];
            this.f32arrayVertices[j+1] = this.pts[i].y + this.vecVerticals[i].y * this.halfWidths[i];
            this.f32arrayVertices[j+2] = this.pts[i].x - this.vecVerticals[i].x * this.halfWidths[i];
            this.f32arrayVertices[j+3] = this.pts[i].y - this.vecVerticals[i].y * this.halfWidths[i];
        }

        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexPositionBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayVertices, gl.STATIC_DRAW);

        gl.bindBuffer(gl.ARRAY_BUFFER, null);
    },

    // 2번 구현방식 : 정점이 매프레임마다 이동할 거리를 미리 계산해 두고, 이값을 증감해준다.
    updateVertexBuffer2: function() {
        // 두번째 구현.

        this.deltaHalfWidths.pop();

        this.curPos = this.followObj.getPosition();
        //var vecV = cc.p(this.curPos.x - this.lastPos.x, this.curPos.y - this.lastPos.y);
        //var vecV = cc.p(this.curPos.y - this.lastPos.y, -this.curPos.x + this.lastPos.x);
        this.vecVNormal = cc.pNormalize(cc.p(this.curPos.y - this.lastPos.y, -this.curPos.x + this.lastPos.x));
        //var delatVecV = cc.pMult(cc.pNormalize(cc.p(this.curPos.y - this.lastPos.y, -this.curPos.x + this.lastPos.x)), this.deltaHalfWidthRate);
        this.deltaHalfWidths.unshift(cc.pMult(this.vecVNormal, this.deltaHalfWidthMax));


        this.lastPos = this.followObj.getPosition();

        for( var i = this.pointsNum-1; i > 0; --i)
        {
            var j = i*4;
            this.f32arrayVertices[j] = this.f32arrayVertices[j-4] - this.deltaHalfWidths[i].x;
            this.f32arrayVertices[j+1] = this.f32arrayVertices[j-3] - this.deltaHalfWidths[i].y;
            this.f32arrayVertices[j+2] = this.f32arrayVertices[j-2] + this.deltaHalfWidths[i].x;
            this.f32arrayVertices[j+3] = this.f32arrayVertices[j-1] + this.deltaHalfWidths[i].y;
        }

        this.f32arrayVertices[0] = this.curPos.x + this.vecVNormal.x * this.startHalfWidth;
        this.f32arrayVertices[1] = this.curPos.y + this.vecVNormal.y * this.startHalfWidth;
        this.f32arrayVertices[2] = this.curPos.x - this.vecVNormal.x * this.startHalfWidth;
        this.f32arrayVertices[3] = this.curPos.y - this.vecVNormal.y * this.startHalfWidth;

        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexPositionBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayVertices, gl.STATIC_DRAW);

        gl.bindBuffer(gl.ARRAY_BUFFER, null);
    },

    // 3번 구현방식 : 정점이 수정을 최소화한 방식이다. 정점으로 트레일의 모양을 정하지 않고 텍스처로 대신한다.
    //             그리고 정점은 이전정점을 그대로 다음 위치로 넘기는 방식이다.
    updateVertexBuffer3: function() {
        // 세번째 구현.

        //this.curPos = this.followObj.getPosition();
        //this.curPos = this.followObj.convertToWorldSpace(cc.p(0,0));
        this.curPos = g_world.convertToNodeSpace(this.followObj.convertToWorldSpace(cc.p(0,0)));
        //var vecV = cc.p(this.curPos.x - this.lastPos.x, this.curPos.y - this.lastPos.y);
        //var vecV = cc.p(this.curPos.y - this.lastPos.y, -this.curPos.x + this.lastPos.x);
        this.vecVNormal = cc.pNormalize(cc.p(this.curPos.y - this.lastPos.y, -this.curPos.x + this.lastPos.x));

        this.lastPos = this.curPos;


        this.f32arrayVertices.set(this.f32arrayVertices.subarray(0, -4), 4);

        this.f32arrayVertices[0] = this.curPos.x + this.vecVNormal.x * this.startHalfWidth;
        this.f32arrayVertices[1] = this.curPos.y + this.vecVNormal.y * this.startHalfWidth;
        this.f32arrayVertices[2] = this.curPos.x - this.vecVNormal.x * this.startHalfWidth;
        this.f32arrayVertices[3] = this.curPos.y - this.vecVNormal.y * this.startHalfWidth;


        gl.bindBuffer(gl.ARRAY_BUFFER, this.squareVertexPositionBuffer);
        gl.bufferData(gl.ARRAY_BUFFER, this.f32arrayVertices, gl.STATIC_DRAW);

        gl.bindBuffer(gl.ARRAY_BUFFER, null);

        this.spriteTemp.setPosition(g_world.convertToNodeSpace(this.followObj.convertToWorldSpace(cc.p(0,0))));
    }

});

cc.GLNode = cc.GLNode || cc.Node.extend({
    ctor:function(){
        this._super();
        this.init();
    },
    init:function(){
        this._renderCmd._needDraw = true;
        this._renderCmd.rendering =  function(ctx){
            cc.kmGLMatrixMode(cc.KM_GL_MODELVIEW);
            cc.kmGLPushMatrix();
            cc.kmGLLoadMatrix(this._stackMatrix);

            this._node.draw(ctx);

            cc.kmGLPopMatrix();
        };
    },
    draw:function(ctx){
        this._super(ctx);
    }
});
