/**
 * Created by kominpyo on 2015. 4. 26..
 */

var g_world;
var g_playLayer;

var PlayLayer = cc.Layer.extend({

    onExit: function() {
        this.actColor.release();
        this.actLandFadeOut.release();

        cc.eventManager.removeAllListeners();
        this._super();
    },

    onEnter: function() {
        this._super();

        // 터치 입력 설정.
        cc.eventManager.addListener( {
            event: cc.EventListener.TOUCH_ALL_AT_ONCE,
            onTouchesBegan: this.onTouchesBegan,
            onTouchesMoved: this.onTouchesMoved,
            onTouchesEnded: this.onTouchesEnded,
            onTouchesCancelled: this.onTouchesCancelled
        }, this);

        // 안드로이드 백버튼 입력 설정.
        if( cc.sys.os == cc.sys.OS_ANDROID ) {
            cc.eventManager.addListener({
                event: cc.EventListener.KEYBOARD,
                onKeyPressed: function (keyCode, event) {
                },
                onKeyReleased: function (keyCode, event) {
                    if (keyCode == 6) {     // 6 : KeyCode::KEY_ESCAPE (안드로이드에서 백버튼 값이다.)
                        g_systemManager.backButtonClicked();
                    }
                }
            }, this);
        }
    },
    ctor: function() {
        this._super();
        g_playLayer = this;

        // 아틀라스 로딩.
        cc.spriteFrameCache.addSpriteFrames(res.AtlasPlayLayer_plist);
        var tex = cc.textureCache.addImage(res.AtlasPlayLayer_png);
        if( cc.sys.isNative ) {
            tex.setAntiAliasTexParameters();    // 안티 앨리어싱 설정.
        }
        else {
            tex.setAliasTexParameters();
        }

        cc.spriteFrameCache.addSpriteFrames(res.AtlasAlias_plist);
        tex = cc.textureCache.addImage(res.AtlasAlias_png);
        tex.setAliasTexParameters();        // 앨리어싱 설정.

        tex = cc.textureCache.addImage(res.White4A_png);
        tex.setAliasTexParameters();        // 앨리어싱 설정.


        // 땅 높이 설정.
        LAND_POS_Y = winSize.height/2 - 185;


        // 게임 플레이에 필요한 노드들 생성.
        // 월드 노드 생성.
        g_world = new cc.Node();
        this.addChild(g_world);
        g_world.setPosition(0, 0);

        // 플레이어 노드 생성.
        this.player = new MoveHPlayer();
        g_world.addChild(this.player, 2);


        // 땅 스프라이트 순차랜덤 인덱스.
        this.seqRandIdxLand = new SequencedRandom(6);

        // 땅 스프라이트.
        var landOffsetY = 15;
        this.land = new cc.Sprite("#Land"+this.seqRandIdxLand.getNextElement()+".png");
        g_world.addChild(this.land, -5);
        //this.land.setScale(285, 1);
        this.land.setAnchorPoint(0.5, 1);
        this.land.setPosition(winCenter.x, LAND_POS_Y+landOffsetY);

        // 땅 색상변화 이펙트 스프라이트.
        //this.landWhite = new cc.Sprite("#White4A.png");
        this.landWhite = new cc.Sprite("#LandWhite.png");
        g_world.addChild(this.landWhite, -4);
        //this.landWhite.setScale(285, 46);
        this.landWhite.setAnchorPoint(0.5, 1);
        this.landWhite.setPosition(winCenter.x, LAND_POS_Y+landOffsetY);
        this.landWhite.visible = false;



        // 배경 매니저 생성.
        this.bgManager = new BgManager();
        this.addChild(this.bgManager, -20);


        // 볼 매니저.
        this.ballManager = new BallManager();
        this.addChild(this.ballManager);

        // 이펙트 매니저.
        this.effectManager = new EffectManager();
        this.addChild(this.effectManager);

        // 충돌관련 변수 설정.
        // 플레이어-아이템 충돌 거리.
        this.colDistSQPlayerItem = (this.player.COL_RADIUS_HEAD + Gain.colRadius) * (this.player.COL_RADIUS_HEAD + Gain.colRadius);


        // 땅 색상변화 액션
        var timeTint = 0.084;
        this.actColor = cc.repeatForever(
            cc.sequence(
                cc.tintTo(timeTint, 255,202,0),
                cc.tintTo(timeTint, 255,80,255),
                cc.tintTo(timeTint, 0,179,255)
            )
        );
        this.actColor.retain();
        // 땅 페이드 아웃 액션.
        this.actLandFadeOut = cc.sequence(
            cc.fadeOut(0.5),
            cc.hide(),
            cc.callFunc(function(sender) {
                sender.stopAllActions();
            }, this)
        );
        this.actLandFadeOut.retain();

    },

    mainMenuToPlay: function() {
        this.player.mainMenuToPlay();
        this.ballManager.mainMenuToPlay();

        // 플레이 중 배경 사운드 재생.
        cc.audioEngine.playMusic(res.SoundClock, true);
    },
    playToPlayEnd: function() {
        this.player.playToPlayEnd();
        this.ballManager.playToPlayEnd();
        inputManager.reset();

        // 플레이 중 배경 사운드 정지.
        cc.audioEngine.stopMusic();
    },
    playEndToMainMenu: function() {
        this.player.playEndToMainMenu();
        this.ballManager.playEndToMainMenu();
        g_effectManager.reset();

        // 땅 스프라이트 랜덤 설정.
        this.land.setSpriteFrame("Land"+this.seqRandIdxLand.getNextElement()+".png");
    },

    update: function(dt) {

        if( g_systemManager.systemState == SystemState.Play ) {
            inputManager.update(dt);

            // 점수(=시간) 증가.
            g_systemManager.addScore(dt);


            this.player.update(dt);

            this.ballManager.update(dt);

            // 충돌체크.
            this.checkCollision();

        }
        else if( g_systemManager.systemState == SystemState.PlayEnd ) {
            // 게임 종료시 이미 생성되어 있는 공들은 움직인다.
            this.ballManager.updateOnPlayEnd(dt);
        }

        this.effectManager.update(dt);
    },

    checkCollision: function() {
        var colPlayerHeadCenter = cc.pAdd(this.player.getPosition(), this.player.COL_CENTER_POINT_HEAD);
        // Player : Ball
        for( var i = g_ballManager.activeObjects.length; i--; ) {
            var ball = g_ballManager.activeObjects[i];
            if( cc.pDistanceSQ(colPlayerHeadCenter, ball.getPosition()) < ball.colDistSQPlayer )
            {
                // 화면 흔들림.
                //g_playScene.shakeScene(0.3, 12);
                g_playScene.shakeScene(0.6, 18);

                // 화면 줌 인/아웃
                g_playScene.zoomScene(0.6, 0.1, 1.1);

                // 붉은 화면외곽 이펙트
                g_guiLayer.runEffectRedScreen();

                // 죽음 이펙트.
                g_effectManager.spawnBlood(colPlayerHeadCenter, ball.getPosition());


                g_systemManager.playToPlayEnd();
                return;
            }
        }

        // Player : Gain
        for( var i = g_guiLayer.activeGains.length; i--; ) {
            var gain = g_guiLayer.activeGains[i];
            if( cc.pDistanceSQ(colPlayerHeadCenter, gain.getPosition()) < this.colDistSQPlayerItem )
            {
                // 플레이어  획득 처리.
                this.player.obtainGain();

                // 아이템 삭제.
                g_guiLayer.activeGains.splice(i, 1);
                gain.collideWithPlayer();


                // 랜덤시간 이후 다음 아이템 생성.
                g_guiLayer.spawnGainWithRandomTerm();

                // 사운드.
                cc.audioEngine.playEffect(res.SoundGetNumber);
            }
        }
    },

    changeLandColor: function() {
        this.landWhite.visible = true;
        this.landWhite.setOpacity(255);
        this.landWhite.runAction(this.actColor);
        this.scheduleOnce(function() {
            this.land.setSpriteFrame("Land"+this.seqRandIdxLand.getNextElement()+".png");

            this.landWhite.runAction(this.actLandFadeOut);
        }, 1);
    },


    // 터치 입력 구현.
    onTouchesBegan: function(touches, event) {
        if( g_systemManager.systemState == SystemState.Play ) {
            for(var i = touches.length; i--; ) {
                inputManager.addTouchInfo(touches[i]);
            }

            // test
            //g_systemManager.playToPlayEnd();
        }
        else if( g_systemManager.systemState == SystemState.PlayEnd ) {
            //g_systemManager.playEndToMainMenu();
        }
        else if( g_systemManager.systemState == SystemState.MainMenu ) {
            if( PButton.enabled == true ) {
                g_systemManager.mainMenuToPlay();
            }
        }

        return true;
    },
    onTouchesMoved: function(touches, event) {
    },
    onTouchesEnded: function(touches, event) {
        for(var i = touches.length; i--; ) {
            inputManager.removeTouchInfo(touches[i]);
        }

    },
    onTouchesCancelled: function(touches, event) {
        event.getCurrentTarget().onTouchesEnded(touches, event);
    }
});

var LAND_POS_Y = 135;   // 135: 가로/세로 화면 비율이 1.5 이상일 경우의 땅 높이.
                        // playLayer.ctor() 에서 해상도에 따라 재설정한다.
